# Copyright 2021 Red Hat.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Designate statuses/actions strings
PENDING = 'PENDING'
COMPLETE = 'COMPLETE'
ERROR = 'ERROR'
DELETED = 'DELETED'
DELETE = 'DELETE'
ACTIVE = 'ACTIVE'
INACTIVE = 'INACTIVE'
UP = 'UP'
CREATE = 'CREATE'
UPDATE = 'UPDATE'
NONE = 'NONE'

# Zone types
PRIMARY_ZONE_TYPE = 'PRIMARY'
SECONDARY_ZONE_TYPE = 'SECONDARY'

# DNS Record types
A = 'A'
AAAA = 'AAAA'
ALIAS = 'ALIAS'
CNAME = 'CNAME'
MX = 'MX'
NS = 'NS'
PTR = 'PTR'
SOA = 'SOA'
SRV = 'SRV'
TXT = 'TXT'
